@@ -599,13 +599,8 @@ static LoadedOpenSsl loadOpenSsl()
 #define QT_SSL_SUFFIX
 #endif // !Q_PROCESSOR_x86_64
 
-#if QT_CONFIG(opensslv11)
-    tryToLoadOpenSslWin32Library(QLatin1String("libssl-1_1" QT_SSL_SUFFIX),
-                                 QLatin1String("libcrypto-1_1" QT_SSL_SUFFIX), result);
-#elif QT_CONFIG(opensslv30)
-    tryToLoadOpenSslWin32Library(QLatin1String("libssl-3" QT_SSL_SUFFIX),
-                                 QLatin1String("libcrypto-3" QT_SSL_SUFFIX), result);
-#endif
+tryToLoadOpenSslWin32Library(QLatin1String("libssl-3" QT_SSL_SUFFIX),
+                             QLatin1String("libcrypto-3" QT_SSL_SUFFIX), result);
 
 #undef QT_SSL_SUFFIX
 
@@ -673,7 +668,7 @@ static LoadedOpenSsl loadOpenSsl()
         libcrypto->unload();
     }
 
-#if !QT_CONFIG(opensslv11) | QT_CONFIG(opensslv30)
+#if QT_CONFIG(opensslv30)
     // first-and-half attempts: for OpenSSL 1.0 try to load some hardcoded sonames:
     // - "1.0.0" is the official upstream one
     // - "1.0.2" is found on some distributions (e.g. Debian) that patch OpenSSL
@@ -703,14 +698,14 @@ static LoadedOpenSsl loadOpenSsl()
     //    attempt, _after_ <bundle>/Contents/Frameworks has been searched.
     //  iOS does not ship a system libssl.dylib, libcrypto.dylib in the first place.
 # if defined(Q_OS_ANDROID)
-    // OpenSSL 1.1.x must be suffixed otherwise it will use the system libcrypto.so libssl.so which on API-21 are OpenSSL 1.0 not 1.1
+    // OpenSSL 3.x.x must be suffixed in order to be able to find the right OpenSSL libraries
     auto openSSLSuffix = [](const QByteArray &defaultSuffix = {}) {
         auto suffix = qgetenv("ANDROID_OPENSSL_SUFFIX");
         if (suffix.isEmpty())
             return defaultSuffix;
         return suffix;
     };
-    static QString suffix = QString::fromLatin1(openSSLSuffix("_1_1"));
+    static QString suffix = QString::fromLatin1(openSSLSuffix("_3"));
     libssl->setFileNameAndVersion(QLatin1String("ssl") + suffix, -1);
     libcrypto->setFileNameAndVersion(QLatin1String("crypto") + suffix, -1);
 # else
