@@ -24,7 +24,9 @@
 #include <QtOpcUa/QOpcUaRange>
 #include <QtOpcUa/QOpcUaXValue>
 #include <QtOpcUa/QOpcUaStructureDefinition>
+#include <QtOpcUa/QOpcUaStructureField>
 #include <QtOpcUa/QOpcUaEnumDefinition>
+#include <QtOpcUa/QOpcUaEnumField>
 #include <QtOpcUa/QOpcUaDiagnosticInfo>
 
 #include <csignal>
@@ -300,6 +302,8 @@ int main()
 
     server.addEventTrigger(testFolder);
 
+    server.addEventHistorian(UA_NODEID_NUMERIC(0, UA_NS0ID_OBJECTSFOLDER));
+
     server.addServerStatusTypeTestNodes(testFolder);
 
     // Add test nodes for the generic type decoder
